@@ -1,37 +1,54 @@
-# coding: utf-8
 """
     cairocffi.tests
     ~~~~~~~~~~~~~~~
 
     Test suite for cairocffi.
 
-    :copyright: Copyright 2013 by Simon Sapin
+    :copyright: Copyright 2013-2019 by Simon Sapin
     :license: BSD, see LICENSE for details.
 
 """
 
-import io
-import gc
-import os
-import sys
-import math
 import array
 import base64
-import shutil
-import tempfile
 import contextlib
+import gc
+import io
+import math
+import os
+import shutil
+import sys
+import tempfile
 
+import pikepdf
 import pytest
 
 import cairocffi
-from . import *
-from .compat import u, pixel
+
+from . import (  # isort:skip
+    PDF_METADATA_AUTHOR, PDF_METADATA_CREATE_DATE, PDF_METADATA_CREATOR,
+    PDF_METADATA_KEYWORDS, PDF_METADATA_MOD_DATE, PDF_METADATA_SUBJECT,
+    PDF_METADATA_TITLE, PDF_OUTLINE_FLAG_BOLD, PDF_OUTLINE_FLAG_OPEN,
+    PDF_OUTLINE_ROOT, SVG_UNIT_PC, SVG_UNIT_PT, SVG_UNIT_PX, SVG_UNIT_USER,
+    TAG_LINK, Context, FontFace, FontOptions, ImageSurface, LinearGradient,
+    Matrix, Pattern, PDFSurface, PSSurface, RadialGradient, RecordingSurface,
+    ScaledFont, SolidPattern, Surface, SurfacePattern, SVGSurface, ToyFontFace,
+    cairo_version, cairo_version_string)
+
+if sys.byteorder == 'little':
+    def pixel(argb):  # pragma: no cover
+        """Convert a 4-byte ARGB string to native-endian."""
+        return argb[::-1]
+else:
+    def pixel(argb):  # pragma: no cover
+        """Convert a 4-byte ARGB string to native-endian."""
+        return argb
 
 
 @contextlib.contextmanager
 def temp_directory():
-    tempdir = tempfile.mkdtemp(u('é'))
-    assert u('é') in tempdir  # Test non-ASCII filenames
+    tempdir = tempfile.mkdtemp('é')
+    assert 'é' in tempdir  # Test non-ASCII filenames
     try:
         yield tempdir
     finally:
@@ -45,7 +62,7 @@ def round_tuple(values):
 def assert_raise_finished(func, *args, **kwargs):
     with pytest.raises(cairocffi.CairoError) as exc:
         func(*args, **kwargs)
-    assert 'SURFACE_FINISHED' in str(exc)
+    assert 'SURFACE_FINISHED' in str(exc) or 'ExceptionInfo' in str(exc)
 
 
 def test_cairo_version():
@@ -82,7 +99,7 @@ def test_image_surface():
     # The default source is opaque black:
     assert context.get_source().get_rgba() == (0, 0, 0, 1)
     context.paint_with_alpha(0.5)
-    assert data.tostring() == pixel(b'\x80\x00\x00\x00') * 200
+    assert data.tobytes() == pixel(b'\x80\x00\x00\x00') * 200
 
 
 def test_image_bytearray_buffer():
@@ -97,9 +114,9 @@ def test_image_bytearray_buffer():
     assert data == pixel(b'\x80\x00\x00\x00') * 200
 
 
+@pytest.mark.xfail(cairo_version() < 11200,
+                   reason='Cairo version too low')
 def test_surface_create_similar_image():
-    if cairo_version() < 11200:
-        pytest.xfail()
     surface = ImageSurface(cairocffi.FORMAT_ARGB32, 20, 30)
     similar = surface.create_similar_image(cairocffi.FORMAT_A8, 4, 100)
     assert isinstance(similar, ImageSurface)
@@ -109,9 +126,9 @@ def test_surface_create_similar_image():
     assert similar.get_height() == 100
 
 
+@pytest.mark.xfail(cairo_version() < 11000,
+                   reason='Cairo version too low')
 def test_surface_create_for_rectangle():
-    if cairo_version() < 11000:
-        pytest.xfail()
     surface = ImageSurface(cairocffi.FORMAT_A8, 4, 4)
     data = surface.get_data()
     assert data[:] == b'\x00' * 16
@@ -183,9 +200,9 @@ def test_target_lifetime():
     assert sys.getrefcount(target) == initial_refcount
 
 
+@pytest.mark.xfail(cairo_version() < 11000,
+                   reason='Cairo version too low')
 def test_mime_data():
-    if cairo_version() < 11000:
-        pytest.xfail()
     surface = PDFSurface(None, 1, 1)
     assert surface.get_mime_data('image/jpeg') is None
     gc.collect()  # Clean up KeepAlive stuff from other tests
@@ -203,15 +220,141 @@ def test_mime_data():
     assert_raise_finished(surface.set_mime_data, 'image/jpeg', None)
 
 
+@pytest.mark.xfail(cairo_version() < 11200,
+                   reason='Cairo version too low')
 def test_supports_mime_type():
-    if cairo_version() < 11200:
-        pytest.xfail()
     # Also test we get actual booleans:
     assert PDFSurface(None, 1, 1).supports_mime_type('image/jpeg') is True
     surface = ImageSurface(cairocffi.FORMAT_A8, 1, 1)
     assert surface.supports_mime_type('image/jpeg') is False
 
 
+@pytest.mark.xfail(cairo_version() < 11400,
+                   reason='Cairo version too low')
+def test_device_scale():
+    surface = PDFSurface(None, 1, 1)
+    assert surface.get_device_scale() == (1, 1)
+    surface.set_device_scale(2, 3)
+    assert surface.get_device_scale() == (2, 3)
+
+
+@pytest.mark.xfail(cairo_version() < 11504,
+                   reason='Cairo version too low')
+def test_metadata():
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 1, 1)
+    surface.set_metadata(PDF_METADATA_TITLE, 'title')
+    surface.set_metadata(PDF_METADATA_SUBJECT, 'subject')
+    surface.set_metadata(PDF_METADATA_CREATOR, 'creator')
+    surface.set_metadata(PDF_METADATA_AUTHOR, 'author')
+    surface.set_metadata(PDF_METADATA_KEYWORDS, 'keywords')
+    surface.set_metadata(PDF_METADATA_CREATE_DATE, '2013-07-21T23:46:00+01:00')
+    surface.set_metadata(PDF_METADATA_MOD_DATE, '2013-07-21T23:46:00Z')
+    surface.finish()
+    pdf = pikepdf.Pdf.open(file_obj)
+    assert pdf.docinfo['/Title'] == "title"
+    assert pdf.docinfo['/Subject'] == "subject"
+    assert pdf.docinfo['/Creator'] == "creator"
+    assert pdf.docinfo['/Author'] == "author"
+    assert pdf.docinfo['/Keywords'] == "keywords"
+    # cairo 1.17.4 adds an apostrophe at the end of dates:
+    # https://gitlab.freedesktop.org/cairo/cairo/-/issues/392
+    assert str(pdf.docinfo['/CreationDate']).startswith("20130721234600+01'00")
+    assert pdf.docinfo['/ModDate'] == "20130721234600Z"
+
+
+@pytest.mark.xfail(cairo_version() < 11504,
+                   reason='Cairo version too low')
+def test_outline():
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 1, 1)
+    outline = surface.add_outline(
+        PDF_OUTLINE_ROOT, 'title 1', 'page=1 pos=[1 1]',
+        PDF_OUTLINE_FLAG_OPEN & PDF_OUTLINE_FLAG_BOLD)
+    surface.add_outline(outline, 'title 2', 'page=1 pos=[1 1]')
+    surface.finish()
+    pdf = pikepdf.Pdf.open(file_obj)
+    outline = pdf.open_outline()
+    assert outline.root[0].title == "title 1"
+    assert outline.root[0].children[0].title == "title 2"
+
+
+@pytest.mark.xfail(cairo_version() < 11504,
+                   reason='Cairo version too low')
+def test_page_label():
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 1, 1)
+    surface.set_page_label('abc')
+    surface.finish()
+    pdf = pikepdf.Pdf.open(file_obj)
+    assert pdf.pages[0].label == "abc"
+
+
+@pytest.mark.xfail(cairo_version() < 11504,
+                   reason='Cairo version too low')
+def test_tag():
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 10, 10)
+    context = Context(surface)
+    context.tag_begin('Document')
+    context.tag_begin(
+        TAG_LINK,
+        attributes='rect=[1 2 4 5] uri=\'https://www.courtbouillon.org/\'')
+    context.set_source_rgba(1, 0, .5, 1)
+    context.rectangle(2, 3, 4, 5)
+    context.fill()
+    context.tag_end(TAG_LINK)
+    context.tag_end('Document')
+    context.show_page()
+    surface.finish()
+    pdf = pikepdf.Pdf.open(file_obj)
+    assert '"/URI": "https://www.courtbouillon.org/"' in str(pdf.objects)
+    assert '"/S": "/Document"' in str(pdf.objects)
+
+
+@pytest.mark.xfail(cairo_version() < 11504,
+                   reason='Cairo version too low')
+def test_thumbnail_size():
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 1, 1)
+    surface.set_thumbnail_size(1, 1)
+    surface.finish()
+    pdf_bytes1 = file_obj.getvalue()
+
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 1, 1)
+    surface.set_thumbnail_size(9, 9)
+    surface.finish()
+    pdf_bytes2 = file_obj.getvalue()
+
+    assert len(pdf_bytes1) < len(pdf_bytes2)
+
+
+@pytest.mark.xfail(cairo_version() < 11510,
+                   reason='Cairo version too low')
+def test_document_unit():
+    surface = SVGSurface(None, 1, 2)
+    assert surface.get_document_unit() in (SVG_UNIT_USER, SVG_UNIT_PT)
+
+    file_obj = io.BytesIO()
+    surface = SVGSurface(file_obj, 1, 2)
+    surface.set_document_unit(SVG_UNIT_PX)
+    assert surface.get_document_unit() == SVG_UNIT_PX
+    surface.finish()
+    pdf_bytes = file_obj.getvalue()
+    assert b'width="1px"' in pdf_bytes
+    assert b'height="2px"' in pdf_bytes
+
+    file_obj = io.BytesIO()
+    surface = SVGSurface(file_obj, 1, 2)
+    surface.set_document_unit(SVG_UNIT_PC)
+    assert surface.get_document_unit() == SVG_UNIT_PC
+    surface.finish()
+    pdf_bytes = file_obj.getvalue()
+    assert b'width="1pc"' in pdf_bytes
+    assert b'height="2pc"' in pdf_bytes
+
+
 def test_png():
     png_bytes = base64.b64decode(
         b'iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAADUlEQVQI12O'
@@ -255,9 +398,9 @@ def test_png():
         surface = ImageSurface.create_from_png(io.BytesIO(b''))
 
 
+@pytest.mark.xfail(cairo_version() < 11000,
+                   reason='Cairo version too low')
 def test_pdf_versions():
-    if cairo_version() < 11000:
-        pytest.xfail()
     assert set(PDFSurface.get_versions()) >= set([
         cairocffi.PDF_VERSION_1_4, cairocffi.PDF_VERSION_1_5])
     assert PDFSurface.version_to_string(cairocffi.PDF_VERSION_1_4) == 'PDF 1.4'
@@ -268,7 +411,7 @@ def test_pdf_versions():
 
     file_obj = io.BytesIO()
     PDFSurface(file_obj, 1, 1).finish()
-    assert file_obj.getvalue().startswith(b'%PDF-1.5')
+    assert file_obj.getvalue().startswith((b'%PDF-1.5', b'%PDF-1.7'))
 
     file_obj = io.BytesIO()
     surface = PDFSurface(file_obj, 1, 1)
@@ -289,10 +432,9 @@ def test_pdf_surface():
             assert fd.read().startswith(b'%PDF')
         with open(filename_bytes, 'rb') as fd:
             assert fd.read().startswith(b'%PDF')
-        pdf_bytes = file_obj.getvalue()
-        assert pdf_bytes.startswith(b'%PDF')
-        assert b'/MediaBox [ 0 0 123 432 ]' in pdf_bytes
-        assert pdf_bytes.count(b'/Type /Page\n') == 1
+        pdf = pikepdf.Pdf.open(file_obj)
+        assert pdf.pages[0]['/MediaBox'] == [0, 0, 123, 432]
+        assert len(pdf.pages) == 1
 
     file_obj = io.BytesIO()
     surface = PDFSurface(file_obj, 1, 1)
@@ -302,11 +444,11 @@ def test_pdf_surface():
     surface.set_size(42, 700)
     context.copy_page()
     surface.finish()
-    pdf_bytes = file_obj.getvalue()
-    assert b'/MediaBox [ 0 0 1 1 ]' not in pdf_bytes
-    assert b'/MediaBox [ 0 0 12 100 ]' in pdf_bytes
-    assert b'/MediaBox [ 0 0 42 700 ]' in pdf_bytes
-    assert pdf_bytes.count(b'/Type /Page\n') == 2
+    pdf = pikepdf.Pdf.open(file_obj)
+    assert '"/MediaBox": [ 0 0 1 1 ]' not in str(pdf.objects)
+    assert pdf.pages[0]['/MediaBox'] == [0, 0, 12, 100]
+    assert pdf.pages[1]['/MediaBox'] == [0, 0, 42, 700]
+    assert len(pdf.pages) == 2
 
 
 def test_svg_surface():
@@ -367,7 +509,7 @@ def test_ps_surface():
     surface.set_eps('')
     assert surface.get_eps() is False
     surface.set_size(10, 12)
-    surface.dsc_comment(u('%%Lorem'))
+    surface.dsc_comment('%%Lorem')
     surface.dsc_begin_setup()
     surface.dsc_comment('%%ipsum')
     surface.dsc_begin_page_setup()
@@ -379,9 +521,9 @@ def test_ps_surface():
     assert b'%%dolor' in ps_bytes
 
 
+@pytest.mark.xfail(cairo_version() < 11000,
+                   reason='Cairo version too low')
 def _recording_surface_common(extents):
-    if cairo_version() < 11000:
-        pytest.xfail()
     surface = ImageSurface(cairocffi.FORMAT_ARGB32, 100, 100)
     empty_pixels = surface.get_data()[:]
     assert empty_pixels == b'\x00' * 40000
@@ -415,17 +557,16 @@ def test_recording_surface():
     assert recorded_pixels == text_pixels
 
 
+@pytest.mark.xfail(cairo_version() < 11200,
+                   reason='Cairo version too low')
 def test_unbounded_recording_surface():
-    if cairo_version() < 11200:
-        # Unbounded recording surfaces do not seem to record anything on 1.10.
-        pytest.xfail()
     text_pixels, recorded_pixels = _recording_surface_common(None)
     assert recorded_pixels == text_pixels
 
 
+@pytest.mark.xfail(cairo_version() < 11200,
+                   reason='Cairo version too low')
 def test_recording_surface_get_extents():
-    if cairo_version() < 11200:
-        pytest.xfail()
     for extents in [None, (0, 0, 140, 80)]:
         surface = RecordingSurface(cairocffi.CONTENT_COLOR_ALPHA, extents)
         assert surface.get_extents() == extents
@@ -865,9 +1006,9 @@ def test_context_clip():
     assert context.clip_extents() == (1, 1, 3, 4)
 
 
+@pytest.mark.xfail(cairo_version() < 11000,
+                   reason='Cairo version too low')
 def test_context_in_clip():
-    if cairo_version() < 11000:
-        pytest.xfail()
     surface = ImageSurface(cairocffi.FORMAT_A8, 4, 4)
     context = Context(surface)
     context.rectangle(1, 1, 2, 2)
@@ -893,7 +1034,7 @@ def test_context_mask():
     context.mask(SurfacePattern(mask_surface))
     o = pixel(b'\x00\x00\x00\x00')
     b = pixel(b'\x80\x00\x00\x00')
-    B = pixel(b'\xFF\x00\x00\x00')
+    B = pixel(b'\xFF\x00\x00\x00')  # noqa: N806
     assert surface.get_data()[:] == (
         B + o + o + o +
         o + b + o + o +
@@ -906,7 +1047,7 @@ def test_context_mask():
     context.mask_surface(mask_surface, surface_x=1, surface_y=2)
     o = pixel(b'\x00\x00\x00\x00')
     b = pixel(b'\x80\x00\x00\x00')
-    B = pixel(b'\xFF\x00\x00\x00')
+    B = pixel(b'\xFF\x00\x00\x00')  # noqa: N806
     assert surface.get_data()[:] == (
         o + o + o + o +
         o + o + o + o +
@@ -925,10 +1066,10 @@ def test_context_font():
     assert context.get_font_matrix().as_tuple() == (14, 0, 0, 14, 0, 0)
 
     context.set_font_size(10)
-    context.select_font_face(b'serif', cairocffi.FONT_SLANT_ITALIC)
+    context.select_font_face(b'@cairo:serif', cairocffi.FONT_SLANT_ITALIC)
     font_face = context.get_font_face()
     assert isinstance(font_face, ToyFontFace)
-    assert font_face.get_family() == 'serif'
+    assert font_face.get_family() == '@cairo:serif'
     assert font_face.get_slant() == cairocffi.FONT_SLANT_ITALIC
     assert font_face.get_weight() == cairocffi.FONT_WEIGHT_NORMAL
 
@@ -945,14 +1086,13 @@ def test_context_font():
     ascent, descent, height, max_x_advance, max_y_advance = (
         context.font_extents())
     # That’s about all we can assume for a default font.
-#    assert height > ascent + descent  # Not even this is true on all fonts
     assert max_x_advance > 0
     assert max_y_advance == 0
     _, _, _, _, x_advance, y_advance = context.text_extents('i' * 10)
     assert x_advance > 0
     assert y_advance == 0
-    context.set_font_face(ToyFontFace(u('monospace'),
-                          weight=cairocffi.FONT_WEIGHT_BOLD))
+    context.set_font_face(
+        ToyFontFace('@cairo:monospace', weight=cairocffi.FONT_WEIGHT_BOLD))
     _, _, _, _, x_advance_mono, y_advance = context.text_extents('i' * 10)
     assert x_advance_mono > x_advance
     assert y_advance == 0
@@ -989,14 +1129,16 @@ def test_scaled_font():
     font = ScaledFont(ToyFontFace())
     font_extents = font.extents()
     ascent, descent, height, max_x_advance, max_y_advance = font_extents
-#    assert height > ascent + descent  # Not even this is true on all fonts
     assert max_x_advance > 0
     assert max_y_advance == 0
     _, _, _, _, x_advance, y_advance = font.text_extents('i' * 10)
     assert x_advance > 0
     assert y_advance == 0
 
-    font = ScaledFont(ToyFontFace('monospace'))
+    font = ScaledFont(ToyFontFace('@cairo:serif'))
+    _, _, _, _, x_advance, y_advance = font.text_extents('i' * 10)
+
+    font = ScaledFont(ToyFontFace('@cairo:monospace'))
     _, _, _, _, x_advance_mono, y_advance = font.text_extents('i' * 10)
     assert x_advance_mono > x_advance
     assert y_advance == 0
@@ -1005,7 +1147,8 @@ def test_scaled_font():
     assert isinstance(font.get_font_face(), FontFace)
 
     font = ScaledFont(
-        ToyFontFace('monospace'), Matrix(xx=20, yy=20), Matrix(xx=3, yy=.5),
+        ToyFontFace('@cairo:monospace'),
+        Matrix(xx=20, yy=20), Matrix(xx=3, yy=.5),
         FontOptions(antialias=cairocffi.ANTIALIAS_BEST))
     assert font.get_font_options().get_antialias() == cairocffi.ANTIALIAS_BEST
     assert font.get_font_matrix().as_tuple() == (20, 0, 0, 20, 0, 0)
@@ -1050,11 +1193,23 @@ def test_font_options():
     assert options_2 == options_1
 
 
+@pytest.mark.xfail(cairo_version() < 11512,
+                   reason='Cairo version too low')
+def test_font_options_variations():
+    options = FontOptions()
+
+    assert options.get_variations() is None
+    options.set_variations('wght 400, wdth 300')
+    assert options.get_variations() == 'wght 400, wdth 300'
+    options.set_variations(None)
+    assert options.get_variations() is None
+
+
 def test_glyphs():
     surface = ImageSurface(cairocffi.FORMAT_ARGB32, 100, 20)
     context = Context(surface)
     font = context.get_scaled_font()
-    text = u('Étt')
+    text = 'Étt'
     glyphs, clusters, is_backwards = font.text_to_glyphs(
         5, 15, text, with_clusters=True)
     assert font.text_to_glyphs(5, 15, text, with_clusters=False) == glyphs
@@ -1064,8 +1219,10 @@ def test_glyphs():
     assert 5 == x1 < x2 < x3
     assert clusters == [(2, 1), (1, 1), (1, 1)]
     assert is_backwards == 0
-    assert round_tuple(font.glyph_extents(glyphs)) == round_tuple(font.text_extents(text))
-    assert round_tuple(font.glyph_extents(glyphs)) == round_tuple(context.glyph_extents(glyphs))
+    assert round_tuple(font.glyph_extents(glyphs)) == (
+        round_tuple(font.text_extents(text)))
+    assert round_tuple(font.glyph_extents(glyphs)) == (
+        round_tuple(context.glyph_extents(glyphs)))
 
     assert context.copy_path() == []
     context.glyph_path(glyphs)
@@ -1110,3 +1267,15 @@ def test_from_null_pointer():
     for class_ in [Surface, Context, Pattern, FontFace, ScaledFont]:
         with pytest.raises(ValueError):
             class_._from_pointer(cairocffi.ffi.NULL, 'unused')
+
+
+@pytest.mark.xfail(cairo_version() < 11800,
+                   reason='Cairo version too low')
+def test_hairline():
+    for extents in [None, (0, 0, 140, 80)]:
+        surface = RecordingSurface(cairocffi.CONTENT_COLOR_ALPHA, extents)
+        ctx = Context(surface)
+        ctx.set_hairline(True)
+        assert(ctx.get_hairline() is True)
+        ctx.set_hairline(False)
+        assert(ctx.get_hairline() is False)
